@@ -30,6 +30,7 @@
 #if defined(__clang__)
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wunused-local-typedef"
+#pragma clang diagnostic ignored "-Winfinite-recursion"
 #endif
 #include <boost/asio.hpp>
 #include <boost/asio/ssl.hpp>
@@ -43,12 +44,15 @@
 #error "Cpp rest SDK requires c++11 smart pointer support from boost"
 #endif
 
-#include "cpprest/details/http_client_impl.h"
+#include "http_client_impl.h"
+#include "cpprest/base_uri.h"
 #include "cpprest/details/x509_cert_utilities.h"
 #include <unordered_set>
 
 using boost::asio::ip::tcp;
 
+#define CRLF std::string("\r\n")
+
 namespace web { namespace http
 {
 namespace client
@@ -68,24 +72,15 @@ enum class httpclient_errorcode_context
     close
 };
 
-class asio_connection_pool;
 class asio_connection
 {
-    friend class asio_connection_pool;
     friend class asio_client;
 public:
-    asio_connection(boost::asio::io_service& io_service, bool start_with_ssl, const std::function<void(boost::asio::ssl::context&)>& ssl_context_callback) :
-    m_socket(io_service),
-    m_ssl_context_callback(ssl_context_callback),
-    m_pool_timer(io_service),
-    m_is_reused(false),
-    m_keep_alive(true)
-    {
-        if (start_with_ssl)
-        {
-            upgrade_to_ssl();
-        }
-    }
+    asio_connection(boost::asio::io_service& io_service)
+        : m_socket(io_service),
+        m_is_reused(false),
+        m_keep_alive(true)
+    {}
 
     ~asio_connection()
     {
@@ -93,14 +88,17 @@ public:
     }
 
     // This simply instantiates the internal state to support ssl. It does not perform the handshake.
-    void upgrade_to_ssl()
+    void upgrade_to_ssl(const std::function<void(boost::asio::ssl::context&)>& ssl_context_callback)
     {
         std::lock_guard<std::mutex> lock(m_socket_lock);
         assert(!is_ssl());
         boost::asio::ssl::context ssl_context(boost::asio::ssl::context::sslv23);
         ssl_context.set_default_verify_paths();
         ssl_context.set_options(boost::asio::ssl::context::default_workarounds);
-        m_ssl_context_callback(ssl_context);
+        if (ssl_context_callback)
+        {
+            ssl_context_callback(ssl_context);
+        }
         m_ssl_stream = utility::details::make_unique<boost::asio::ssl::stream<boost::asio::ip::tcp::socket &>>(m_socket, ssl_context);
     }
 
@@ -124,11 +122,6 @@ public:
         return error;
     }
 
-    void cancel_pool_timer()
-    {
-        m_pool_timer.cancel();
-    }
-
     bool is_reused() const { return m_is_reused; }
     void set_keep_alive(bool keep_alive) { m_keep_alive = keep_alive; }
     bool keep_alive() const { return m_keep_alive; }
@@ -213,22 +206,12 @@ public:
         }
     }
 
-private:
-    template <typename TimeoutHandler>
-    void start_pool_timer(int timeout_secs, const TimeoutHandler &handler)
-    {
-        m_pool_timer.expires_from_now(boost::posix_time::milliseconds(timeout_secs * 1000));
-        m_pool_timer.async_wait(handler);
-    }
-
     void start_reuse()
     {
-        cancel_pool_timer();
         m_is_reused = true;
     }
 
-    void handle_pool_timer(const boost::system::error_code& ec);
-
+private:
     // Guards concurrent access to socket/ssl::stream. This is necessary
     // because timeouts and cancellation can touch the socket at the same time
     // as normal message processing.
@@ -236,119 +219,165 @@ private:
     tcp::socket m_socket;
     std::unique_ptr<boost::asio::ssl::stream<tcp::socket &> > m_ssl_stream;
 
-    std::function<void(boost::asio::ssl::context&)> m_ssl_context_callback;
-
-    boost::asio::deadline_timer m_pool_timer;
     bool m_is_reused;
     bool m_keep_alive;
 };
 
-class asio_connection_pool
+/// <summary>Implements a connection pool with adaptive connection removal</summary>
+/// <remarks>
+/// The timeout mechanism is based on the `uint64_t m_epoch` member. Every 30 seconds,
+/// the lambda in `start_epoch_interval` fires, triggering the cleanup of any
+/// connections that have resided in the pool since the last cleanup phase's epoch.
+///
+/// This works because the `m_connections` member functions is used in LIFO order.
+/// LIFO usage guarantees that the elements remain sorted based on epoch number,
+/// since the highest epoch is always removed and on insertion the next monotonically
+/// increasing epoch is used.
+///
+/// During the cleanup phase, connections are removed starting with the oldest. This
+/// ensures that if a high intensity workload is followed by a low intensity workload,
+/// the connection pool will correctly adapt to the low intensity workload.
+///
+/// Specifically, the following code will eventually result in a maximum of one pooled
+/// connection regardless of the initial number of pooled connections:
+/// <code>
+///   while(1)
+///   {
+///     auto conn = pool.acquire();
+///     if (!conn) conn = new_conn();
+///     pool.release(conn);
+///   }
+/// </code>
+/// </remarks>
+class asio_connection_pool : public std::enable_shared_from_this<asio_connection_pool>
 {
 public:
-
-    asio_connection_pool(boost::asio::io_service& io_service, bool start_with_ssl, const std::chrono::seconds &idle_timeout, const std::function<void(boost::asio::ssl::context&)> &ssl_context_callback) :
-    m_io_service(io_service),
-    m_timeout_secs(static_cast<int>(idle_timeout.count())),
-    m_start_with_ssl(start_with_ssl),
-    m_ssl_context_callback(ssl_context_callback)
+    asio_connection_pool() : m_pool_epoch_timer(crossplat::threadpool::shared_instance().service())
     {}
 
-    ~asio_connection_pool()
+    std::shared_ptr<asio_connection> acquire()
     {
-        std::lock_guard<std::mutex> lock(m_connections_mutex);
-        // Cancel the pool timer for all connections.
-        for (auto& connection : m_connections)
-        {
-            connection->cancel_pool_timer();
-        }
-    }
+        std::lock_guard<std::mutex> lock(m_lock);
 
-    void release(const std::shared_ptr<asio_connection> &connection)
-    {
-        if (connection->keep_alive() && (m_timeout_secs > 0))
-        {
-            connection->cancel();
-
-            std::lock_guard<std::mutex> lock(m_connections_mutex);
-            // This will destroy and remove the connection from pool after the set timeout.
-            // We use 'this' because async calls to timer handler only occur while the pool exists.
-            connection->start_pool_timer(m_timeout_secs, boost::bind(&asio_connection_pool::handle_pool_timer, this, boost::asio::placeholders::error, connection));
-            m_connections.push_back(connection);
-        }
-        // Otherwise connection is not put to the pool and it will go out of scope.
-    }
-
-    std::shared_ptr<asio_connection> obtain()
-    {
-        std::unique_lock<std::mutex> lock(m_connections_mutex);
         if (m_connections.empty())
-        {
-            lock.unlock();
+            return nullptr;
 
-            // No connections in pool => create a new connection instance.
-            return std::make_shared<asio_connection>(m_io_service, m_start_with_ssl, m_ssl_context_callback);
-        }
-        else
-        {
-            // Reuse connection from pool.
-            auto connection = m_connections.back();
-            m_connections.pop_back();
-            lock.unlock();
+        auto conn = std::move(m_connections.back().second);
+        m_connections.pop_back();
+        conn->start_reuse();
+        return conn;
+    }
 
-            connection->start_reuse();
-            return connection;
+    void release(const std::shared_ptr<asio_connection>& connection)
+    {
+        connection->cancel();
+
+        if (!connection->keep_alive())
+            return;
+
+        std::lock_guard<std::mutex> lock(m_lock);
+        if (!is_timer_running)
+        {
+            start_epoch_interval(shared_from_this());
+            is_timer_running = true;
         }
+
+        m_epoch++;
+        m_connections.emplace_back(m_epoch, std::move(connection));
     }
 
 private:
-
-    // Using weak_ptr here ensures bind() to this handler will not prevent the connection object from going out of scope.
-    void handle_pool_timer(const boost::system::error_code& ec, const std::weak_ptr<asio_connection> &connection)
+    // Note: must be called under m_lock
+    static void start_epoch_interval(const std::shared_ptr<asio_connection_pool>& pool)
     {
-        if (!ec)
+        _ASSERTE(pool.get() != nullptr);
+
+        auto& self = *pool;
+        std::weak_ptr<asio_connection_pool> weak_pool = pool;
+
+        self.m_prev_epoch = self.m_epoch;
+        pool->m_pool_epoch_timer.expires_from_now(boost::posix_time::seconds(30));
+        pool->m_pool_epoch_timer.async_wait([weak_pool](const boost::system::error_code& ec)
         {
-            auto connection_shared = connection.lock();
-            if (connection_shared)
+            if (ec)
+                return;
+
+            auto pool = weak_pool.lock();
+            if (!pool)
+                return;
+            auto& self = *pool;
+
+            std::lock_guard<std::mutex> lock(self.m_lock);
+            if (self.m_prev_epoch == self.m_epoch)
             {
-                std::lock_guard<std::mutex> lock(m_connections_mutex);
-                const auto &iter = std::find(m_connections.begin(), m_connections.end(), connection_shared);
-                if (iter != m_connections.end())
-                {
-                    m_connections.erase(iter);
-                }
+                self.m_connections.clear();
+                self.is_timer_running = false;
+                return;
             }
-        }
+            else
+            {
+                auto prev_epoch = self.m_prev_epoch;
+                auto erase_end = std::find_if(self.m_connections.begin(), self.m_connections.end(),
+                    [prev_epoch](std::pair<uint64_t, std::shared_ptr<asio_connection>>& p)
+                {
+                    return p.first > prev_epoch;
+                });
+
+                self.m_connections.erase(self.m_connections.begin(), erase_end);
+                start_epoch_interval(pool);
+            }
+        });
     }
 
-    boost::asio::io_service& m_io_service;
-    const int m_timeout_secs;
-    const bool m_start_with_ssl;
-    const std::function<void(boost::asio::ssl::context&)>& m_ssl_context_callback;
-    std::vector<std::shared_ptr<asio_connection> > m_connections;
-    std::mutex m_connections_mutex;
+    std::mutex m_lock;
+    std::deque<std::pair<uint64_t, std::shared_ptr<asio_connection>>> m_connections;
+
+    uint64_t m_epoch = 0;
+    uint64_t m_prev_epoch = 0;
+    bool is_timer_running = false;
+    boost::asio::deadline_timer m_pool_epoch_timer;
 };
 
-
-
-class asio_client : public _http_client_communicator, public std::enable_shared_from_this<asio_client>
+class asio_client final : public _http_client_communicator
 {
 public:
-    asio_client(http::uri address, http_client_config client_config)
-    : _http_client_communicator(std::move(address), std::move(client_config))
-    , m_pool(crossplat::threadpool::shared_instance().service(),
-             base_uri().scheme() == "https" && !_http_client_communicator::client_config().proxy().is_specified(),
-             std::chrono::seconds(30), // Unused sockets are kept in pool for 30 seconds.
-             this->client_config().get_ssl_context_callback()) 
-    , m_resolver(crossplat::threadpool::shared_instance().service())
+    asio_client(http::uri&& address, http_client_config&& client_config)
+        : _http_client_communicator(std::move(address), std::move(client_config))
+        , m_resolver(crossplat::threadpool::shared_instance().service())
+        , m_pool(std::make_shared<asio_connection_pool>())
+        , m_start_with_ssl(base_uri().scheme() == "https" && !this->client_config().proxy().is_specified())
     {}
 
     void send_request(const std::shared_ptr<request_context> &request_ctx) override;
 
     unsigned long open() override { return 0; }
 
-    asio_connection_pool m_pool;
+    void release_connection(std::shared_ptr<asio_connection>& conn)
+    {
+        m_pool->release(conn);
+    }
+    std::shared_ptr<asio_connection> obtain_connection()
+    {
+        std::shared_ptr<asio_connection> conn = m_pool->acquire();
+
+        if (conn == nullptr)
+        {
+            // Pool was empty. Create a new connection
+            conn = std::make_shared<asio_connection>(crossplat::threadpool::shared_instance().service());
+            if (m_start_with_ssl)
+                conn->upgrade_to_ssl(this->client_config().get_ssl_context_callback());
+        }
+
+        return conn;
+    }
+
+    virtual pplx::task<http_response> propagate(http_request request) override;
+
+public:
     tcp::resolver m_resolver;
+private:
+    const std::shared_ptr<asio_connection_pool> m_pool;
+    const bool m_start_with_ssl;
 };
 
 class asio_context : public request_context, public std::enable_shared_from_this<asio_context>
@@ -372,13 +401,13 @@ public:
     {
         m_timer.stop();
         // Release connection back to the pool. If connection was not closed, it will be put to the pool for reuse.
-        std::static_pointer_cast<asio_client>(m_http_client)->m_pool.release(m_connection);
+        std::static_pointer_cast<asio_client>(m_http_client)->release_connection(m_connection);
     }
 
     static std::shared_ptr<request_context> create_request_context(std::shared_ptr<_http_client_communicator> &client, http_request &request)
     {
         auto client_cast(std::static_pointer_cast<asio_client>(client));
-        auto connection(client_cast->m_pool.obtain());
+        auto connection(client_cast->obtain_connection());
         auto ctx = std::make_shared<asio_context>(client, request, connection);
         ctx->m_timer.set_ctx(std::weak_ptr<asio_context>(ctx));
         return ctx;
@@ -436,6 +465,12 @@ public:
                 m_context->m_timer.reset();
                 auto endpoint = *endpoints;
                 m_context->m_connection->async_connect(endpoint, boost::bind(&ssl_proxy_tunnel::handle_tcp_connect, shared_from_this(), boost::asio::placeholders::error, ++endpoints));
+
+                // TODO: refactor all interactions with the timeout_timer to avoid racing
+                if (m_context->m_timer.has_timedout())
+                {
+                    m_context->m_connection->close();
+                }
             }
         }
 
@@ -455,7 +490,7 @@ public:
                 m_context->m_timer.reset();
                 //// Replace the connection. This causes old connection object to go out of scope.
                 auto client = std::static_pointer_cast<asio_client>(m_context->m_http_client);
-                m_context->m_connection = client->m_pool.obtain();
+                m_context->m_connection = client->obtain_connection();
 
                 auto endpoint = *endpoints;
                 m_context->m_connection->async_connect(endpoint, boost::bind(&ssl_proxy_tunnel::handle_tcp_connect, shared_from_this(), boost::asio::placeholders::error, ++endpoints));
@@ -474,7 +509,7 @@ public:
             {
                 m_context->report_error("Failed to send connect request to proxy.", err, httpclient_errorcode_context::writebody);
             }
-	    }
+        }
     
         void handle_status_line(const boost::system::error_code& ec)
         {
@@ -501,9 +536,9 @@ public:
                     m_context->report_error(err_ss.str(), ec, httpclient_errorcode_context::readheader);
                     return;
                 }
-                
-                m_context->m_connection->upgrade_to_ssl();
-                
+
+                m_context->m_connection->upgrade_to_ssl(m_context->m_http_client->client_config().get_ssl_context_callback());
+
                 m_ssl_tunnel_established(m_context);
             }
             else
@@ -630,7 +665,12 @@ public:
             {
                 extra_headers.append(ctx->generate_basic_proxy_auth_header());
             }
-                
+
+            if (ctx->m_http_client->client_config().credentials().is_set())
+            {
+                extra_headers.append(ctx->generate_basic_auth_header());
+            }
+
             // Check user specified transfer-encoding.
             std::string transferencoding;
             if (ctx->m_request.headers().match(header_names::transfer_encoding, transferencoding) && transferencoding == "chunked")
@@ -656,7 +696,7 @@ public:
                 extra_headers.append(": no-cache" + CRLF);
             }
                 
-            request_stream << flatten_http_headers(ctx->m_request.headers());
+            request_stream << ::web::http::details::flatten_http_headers(ctx->m_request.headers());
             request_stream << extra_headers;
             // Enforce HTTP connection keep alive (even for the old HTTP/1.0 protocol).
             request_stream << "Connection: Keep-Alive" << CRLF << CRLF;
@@ -729,6 +769,23 @@ public:
     }
 
 private:
+    utility::string_t generate_basic_auth_header()
+    {
+        utility::string_t header;
+
+        header.append(header_names::authorization);
+        header.append(": Basic ");
+
+        auto credential_str = web::details::plaintext_string(new ::utility::string_t(m_http_client->client_config().credentials().username()));
+        credential_str->append(":");
+        credential_str->append(*m_http_client->client_config().credentials().decrypt());
+
+        std::vector<unsigned char> credentials_buffer(credential_str->begin(), credential_str->end());
+
+        header.append(utility::conversions::to_base64(credentials_buffer));
+        header.append(CRLF);
+        return header;
+    }
 
     utility::string_t generate_basic_proxy_auth_header()
     {
@@ -754,7 +811,7 @@ private:
         long errorcodeValue = ec.value();
 
         // map timer cancellation to time_out
-        if (ec == boost::system::errc::operation_canceled && m_timer.has_timedout())
+        if (m_timer.has_timedout())
         {
             errorcodeValue = make_error_code(std::errc::timed_out).value();
         }
@@ -808,7 +865,7 @@ private:
         {
             // Replace the connection. This causes old connection object to go out of scope.
             auto client = std::static_pointer_cast<asio_client>(m_http_client);
-            m_connection = client->m_pool.obtain();
+            m_connection = client->obtain_connection();
 
             auto endpoint = *endpoints;
             m_connection->async_connect(endpoint, boost::bind(&asio_context::handle_connect, shared_from_this(), boost::asio::placeholders::error, ++endpoints));
@@ -826,6 +883,12 @@ private:
             m_timer.reset();
             auto endpoint = *endpoints;
             m_connection->async_connect(endpoint, boost::bind(&asio_context::handle_connect, shared_from_this(), boost::asio::placeholders::error, ++endpoints));
+
+            // TODO: refactor all interactions with the timeout_timer to avoid racing
+            if (m_timer.has_timedout())
+            {
+                m_connection->close();
+            }
         }
     }
 
@@ -1123,7 +1186,7 @@ private:
             if (colon != std::string::npos)
             {
                 auto name = header.substr(0, colon);
-                auto value = header.substr(colon + 2, header.size() - (colon + 3)); // also exclude '\r'
+                auto value = header.substr(colon + 1, header.size() - colon - 2);
                 boost::algorithm::trim(name);
                 boost::algorithm::trim(value);
 
@@ -1429,7 +1492,7 @@ private:
 #else
         std::chrono::microseconds m_duration;
 #endif
-        timer_state m_state;
+        std::atomic<timer_state> m_state;
         std::weak_ptr<asio_context> m_ctx;
         boost::asio::steady_timer m_timer;
     };
@@ -1446,22 +1509,9 @@ private:
 };
 
 
-
-http_network_handler::http_network_handler(const uri &base_uri, const http_client_config &client_config) :
-    m_http_client_impl(std::make_shared<asio_client>(base_uri, client_config))
-{}
-
-pplx::task<http_response> http_network_handler::propagate(http_request request)
+std::shared_ptr<_http_client_communicator> create_platform_final_pipeline_stage(uri&& base_uri, http_client_config&& client_config)
 {
-    auto context = details::asio_context::create_request_context(m_http_client_impl, request);
-
-    // Use a task to externally signal the final result and completion of the task.
-    auto result_task = pplx::create_task(context->m_request_completion);
-
-    // Asynchronously send the response with the HTTP client implementation.
-    m_http_client_impl->async_send_request(context);
-
-    return result_task;
+    return std::make_shared<asio_client>(std::move(base_uri), std::move(client_config));
 }
 
 void asio_client::send_request(const std::shared_ptr<request_context> &request_ctx)
@@ -1488,4 +1538,18 @@ void asio_client::send_request(const std::shared_ptr<request_context> &request_c
     ctx->start_request();
 }
 
+pplx::task<http_response> asio_client::propagate(http_request request)
+{
+    auto self = std::static_pointer_cast<_http_client_communicator>(shared_from_this());
+    auto context = details::asio_context::create_request_context(self, request);
+
+    // Use a task to externally signal the final result and completion of the task.
+    auto result_task = pplx::create_task(context->m_request_completion);
+
+    // Asynchronously send the response with the HTTP client implementation.
+    this->async_send_request(context);
+
+    return result_task;
+}
+
 }}}} // namespaces
