@@ -63,7 +63,8 @@ void CreateException(const Token &tk, const utility::string_t &message)
 {
     utility::ostringstream_t os;
     os << _XPLATSTR("* Line ") << tk.start.m_line << _XPLATSTR(", Column ") << tk.start.m_column << _XPLATSTR(" Syntax error: ") << message;
-    throw web::json::json_exception(os.str().c_str());
+    utility::string_t osStr = os.str();
+    throw web::json::json_exception(osStr.c_str());
 }
 
 template <typename Token>
