@@ -24,10 +24,11 @@
 ****/
 
 #include "stdafx.h"
-#include "cpprest/details/x509_cert_utilities.h"
 
 #if !defined(CPPREST_EXCLUDE_WEBSOCKETS)
 
+#include "cpprest/details/x509_cert_utilities.h"
+
 // Force websocketpp to use C++ std::error_code instead of Boost.
 #define _WEBSOCKETPP_CPP11_SYSTEM_ERROR_
 #if defined(__GNUC__)
@@ -61,6 +62,26 @@
 
 #endif /* __GNUC__ */
 
+// This is a hack to avoid memory leak reports from the debug MSVC CRT for all
+// programs using the library: ASIO calls SSL_library_init() which calls
+// SSL_COMP_get_compression_methods(), which allocates some heap memory and the
+// only way to free it later is to call SSL_COMP_free_compression_methods(),
+// but this function is unaccessible from the application code as OpenSSL is
+// statically linked into the C++ REST SDK DLL. So, just to be nice, call it
+// here ourselves -- even if the real problem is in ASIO (up to v1.60.0).
+#if defined(_WIN32) && !defined(NDEBUG)
+
+#include <openssl/ssl.h>
+static struct ASIO_SSL_memory_leak_suppress
+{
+    ~ASIO_SSL_memory_leak_suppress()
+    {
+        ::SSL_COMP_free_compression_methods();
+    }
+} ASIO_SSL_memory_leak_suppressor;
+
+#endif /* _WIN32 && !NDEBUG */
+
 using websocketpp::lib::placeholders::_1;
 using websocketpp::lib::placeholders::_2;
 using websocketpp::lib::bind;
@@ -159,7 +180,14 @@ public:
                 auto sslContext = websocketpp::lib::shared_ptr<boost::asio::ssl::context>(new boost::asio::ssl::context(boost::asio::ssl::context::sslv23));
                 sslContext->set_default_verify_paths();
                 sslContext->set_options(boost::asio::ssl::context::default_workarounds);
-                sslContext->set_verify_mode(boost::asio::ssl::context::verify_peer);
+                if (m_config.validate_certificates())
+                {
+                    sslContext->set_verify_mode(boost::asio::ssl::context::verify_peer);
+                }
+                else
+                {
+                    sslContext->set_verify_mode(boost::asio::ssl::context::verify_none);
+                }
 
 #if defined(__APPLE__) || (defined(ANDROID) || defined(__ANDROID__)) || defined(_WIN32)
                 m_openssl_failed = false;
