@@ -304,6 +304,33 @@ TEST(TestTasks_void_tasks_default_construction)
     }
 }
 
+TEST(TestTasks_movable_then)
+{
+#ifndef _MSC_VER
+    // create movable only type
+    struct A
+    {
+        A() = default;
+        A(A&&) = default;
+        A& operator=(A&&) = default;
+
+        // explicitly delete copy functions
+        A(const A&) = delete;
+        A& operator=(const A&) = delete;
+
+        char operator()(int)
+        {
+            return 'c';
+        }
+    } a;
+
+    task<int> task = create_task([]{ return 2; });
+    auto f = task.then(std::move(a));
+
+    IsTrue(f.get() == 'c', L".then should be able to work with movable functors");
+#endif // _MSC_VER
+}
+
 TEST(TestTasks_constant_this)
 {
 #ifdef _MSC_VER
